<!DOCTYPE html>
<html lang="en">
  <head>
    <meta charset="utf-8" />
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <meta name="robots" content="noindex">
    <link rel="stylesheet" href="https://unpkg.com/missing.css@1.0.9/dist/missing.min.css">
    <title>django-fragments sample</title>
  </head>
  <body class="container" hx-headers='{"X-CSRFToken": "{{ csrf_token }}"}'>
    {% include './_nav.html' %}
    <main>
      {% block content %}
      {% endblock content %}
    </main>
    {% block footer %}
      {% include './msg.html' %}
    {% endblock footer %}
  </body>
  <script src="https://unpkg.com/htmx.org@1.9.2"></script>
  <script src="https://unpkg.com/hyperscript.org@0.9.8"></script>
</html>
